install.packages(c("rvest","dplyr","stringr")) # 安装指定包

library(rvest) # 网页爬取包
library(dplyr) # 管道函数操作包
library(stringr) # 字符串处理包

num <- 0 # 为文本文档编号，预先设置“num”变量

for (number in c(1:500)){
  url <- sprintf("https://www.chinadaily.com.cn/china/59b8d010a3108c54ed7dfc23/page_%s.html",number) # 通过for循环和sprintf格式化输出实现翻页
  
  # 识别一级网页上的每一条新闻，定位到每一条新闻的标题（“<h4>”），找到每一条新闻对应的二级网页超链接（“<a href="">”）
  url_all <- url %>% read_html(encoding = "UTF-8") %>% html_nodes("div[class=main_art]") %>% 
    html_nodes("h4") %>% html_nodes("a") %>% html_attr("href")
  
  # 遍历访问二级网页的超链接
  for (url_2 in url_all){
    url_2 <- sprintf("https://%s",str_sub(url_2,start=3))
    url_2_page <- url_2 %>% read_html(encoding = "UTF-8") %>% html_nodes("div[class=lft_art]")
    
    # 爬取新闻标题并清洗
    title <- url_2_page %>% html_nodes("h1") %>% html_text()
    title <- gsub("\\s+"," ",title)
    title <- str_trim(title, side="both")
    
    # 爬取新闻元信息并清洗
    meta <- url_2_page %>% html_nodes("div[class=info]") %>% html_text()
    meta <- gsub("\\s+"," ",meta)
    meta <- str_trim(meta, side="both")
    
    # 爬取新闻内容并清洗
    content <- url_2_page %>% html_nodes("div[id=Content]") %>% html_nodes("p") %>% html_text()
    content = str_c(content,collapse=' ')
    content <- gsub("\\s+"," ",content)
    content <- str_trim(content, side="both")
    
    # 通过正则表达式搜索“现代化”的相应关键词，出现次数在3次及以上，则按照“CNMOD001.txt”的名称形式创建独立文本文档，并将爬取内容以XML格式存储于其中
    news <- sprintf("<seg>\n<title>%s</title>\n<meta>%s</meta>\n<content>%s</content>\n</seg>",title,meta,content)
    match_word <- str_extract_all(tolower(news),"modernization|modernizations|modernize|modernizes|modernized|modernizing")
    fre <- length(as.vector(unlist(match_word)))
    if (fre >1){
      num <- num + 1
      num_1 <- formatC(num, flag = '0', width = 3)
      fileout <- file(sprintf("CNMOD%s.txt",num_1),"w")
      print(news)
      write(news,fileout)}
  }
}