install.packages(c("rvest","dplyr","stringr","RSelenium")) # 安装指定包
# 在R语言的文件路径下，通过CMD终端运行java -jar selenium-server-standalone-4.0.0-alpha-2.jar（下载链接“http://selenium-release.storage.googleapis.com/index.html”）
# 将对应谷歌浏览器版本的“chromedriver.exe”放置于R语言文件路径下（下载链接“https://registry.npmmirror.com/binary.html?path=chromedriver/”）
# 如无对应的版本（以上下载链接仅更新到114.0.5735.90版本），请打开谷歌浏览器，更新至最新版本（chrome-设置-关于chrome），并前往网站“https://googlechromelabs.github.io/chrome-for-testing/#stable”下载对应版本的chromedriver

library(rvest) # 网页爬取包
library(RSelenium) # 动态网页爬取包（模拟浏览器）
library(dplyr) # 管道函数操作包
library(stringr) # 字符串处理包

# 配置谷歌模拟浏览器
remDr <- remoteDriver(remoteServerAddr='localhost',port=4444L,browserName='chrome')
remDr_2 <- remoteDriver(remoteServerAddr='localhost',port=4444L,browserName='chrome')

# 基于“RSelenium”以谷歌浏览器访问“新华网”微博主页
url <- "https://m.weibo.cn/u/2810373291" # 新华网微博主页
remDr$open(silent = TRUE)
Sys.sleep(3) # 系统休息3秒，等待页面加载
remDr$navigate(url) # 以谷歌浏览器访问指定网址

all_header <- c() # 为去重，预先设置“all_header”变量
num <- 0 # 为文本文档编号，预先设置“num”变量

repeat{ # 通过“repeat{}”实现循环
  
  Sys.sleep(2) # 每一次循环加载，休息2秒，防止微博反爬机制
  
  # “remDr$getPageSource()[[1]][1]”将页面的Json源码转换为HTML源码，通过“Rvest”包中的“read_html（）”读取HTML源码并存储于向量“page”中
  page <- read_html(remDr$getPageSource()[[1]][1]) 
  
  # 根据标签"div[class=card-main]"找到当前页面的每一条微博
  nodes <- page %>% html_nodes("div[class=card-main]")
  
  for (web in nodes){ # 循环每一条微博
    
    # 爬取博文元数据（“<header>”标签），存储于向量“header”中
    header <- web %>% html_nodes("header") %>% html_text()
    # 爬取博文内容（展开前）（“<div class="weibo-text">”标签），存储于向量“content”中
    content <- web %>% html_nodes("div[class=weibo-text]") %>% html_text()
    
    # 通过向量“header”和“content”的长度是否为零判断是否爬取成功
    if (length(header)==0){next}else if (length(content)==0){next}else{
      # 如果“header”不在“all_header”中，则不重复，并添加至“all_header”中
      if (header %in% all_header){next}else{
        all_header <- c(all_header,header)
        
        tryCatch({ # 如出现错误，循环不中断
          
          #如果存在“...全文”，则爬取存有博文（展开后）的二级网页
          if (grepl("...全文",str_c(content[1]))==TRUE){
            hrefs <- web %>% html_nodes("a") %>% html_attr("href")
            
            # 访问二级网页，爬取博文全文
            url_2 <- sprintf("https://m.weibo.cn%s",hrefs[length(hrefs)]) # 为存有微博全文的二级网页超链接添加标头
            remDr_2$open(silent = TRUE) # 打开remDr_2浏览器
            remDr_2$navigate(url_2) # 通过remDr_2浏览器访问页面
            page_2 <- read_html(remDr_2$getPageSource()[[1]][1]) # 读取页面
            content <- page_2 %>% html_nodes("div[class=weibo-text]") %>% html_text() # 爬取博文全文，用博文（展开后）替换向量“content”中的博文（展开前）
            remDr_2$closeWindow()} # 关闭remDr_2浏览器
            
            # 对博文元数据和内容进行清洗
            content <- gsub("\\s+"," ",content)
            content <- str_trim(content, side="both")
            header <- gsub("\\s+"," ",header)
            header <- str_trim(header, side="both")
            
            # 以XML格式将每条爬取的微博写入独立的文本文档
            weibo <- sprintf("<seg>\n<meta>%s</meta>\n<content>%s</content>\n</seg>\n",header,content) # 格式化输出XML格式
            num <- num + 1 # 为文本文档编号
            num_1 <- formatC(num, flag = '0', width = 3) # 为编号“Add Lead”，如将“1”转化为“001”
            fileout <- file(sprintf("WeiboXH%s.txt",num_1),"w") # 创建文本文档，并以如“WeiboXH001.txt”的形式命名
            print(weibo) # 输出爬取内容
            write(weibo,fileout) # 将爬取内容写入独立的文本文档
            },
            # 如果“tryCatch”下遇到bug，则运行以下代码
            error=function(e) {
              print("error")
              remDr_2$closeWindow()}) 
            
        }}}
    
    # 当前页面爬取完成后，则向下滚动页面加载新内容，重新开始”repeat{}”
    remDr$executeScript("window.scrollTo(0,document.body.scrollHeight);", list(remDr$findElement("css","body")))       
  }